<?php
/**
 * @package     Class Overrider plugin (Presentation at JoomlaDay 2012 Greece by JoomlaFans.gr - Zgouletas Stergios)
 * @version     2.0 (J3-J4)
 * @author      Stergios Zgouletas <info@web-expert.gr>
 * @link        http://www.web-expert.gr
 * @copyright   Copyright (C) 2010-2023 Web-Expert.gr All Rights Reserved
 * @license     http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

defined('_JEXEC') or die('Restricted access');
use Joomla\CMS\Factory as JFactory; 
use Joomla\CMS\Plugin\CMSPlugin as JPlugin;
use Joomla\String\StringHelper as JString;
use Joomla\CMS\Filter\InputFilter as JFilterInput;

class plgSystemClassoverrider extends JPlugin
{
	protected $app;
	
	function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
		if(!$this->app) $this->app=JFactory::getApplication();
	}

	function onAfterSessionStart()
	{
		$this->loadOverrides(__FUNCTION__);
	}
	
	function onAfterRoute()
	{
		$this->loadOverrides(__FUNCTION__);
	}
	
	function onAfterInitialise()
	{
		$this->loadOverrides(__FUNCTION__);
	}
	
	function loadOverrides($event)
	{
		#Vars
		$ClassPaths=$this->params->get('classpath','');
		$debug=(int)$this->params->get('debug',0);
		$env=$this->app->isClient('administrator')?'admin':'site';
		
		#Check
		if(empty($ClassPaths)) return;
		$classesToOverride =@explode("\n",$ClassPaths);
	    
		$html='<div><b>Class Overrider Debug: '.$event.'</b><br/><pre>';
	
		foreach($classesToOverride as $cPaths)
		{
			$classDetails=@explode('|',trim($cPaths),2); 
			$params=$this->getParams($classDetails[0]);
			
			$allowLoad=1;
			foreach($params as $key=>$val)
			{			
				if(in_array($key,array('path')) || empty($val)) continue;
				
				switch($key)
				{
					case 'env':
						$compare=$env;
						$allowLoad=($val=='both' || $val==$env)?1:0;
						break;
					case 'option':
					case 'compo':
						$compare=$this->getVar('option',null);
						$allowLoad=($val=='force' || $val==$compare)?1:0;
						break;
					case 'event':
						$compare=$event;
						$allowLoad=$val==$event?1:0;
						break;
					default:
						$compare=$this->getVar($key,null);
						$allowLoad=$val==$compare?1:0;					
				}
				
				if($debug)
				{					
					$html.='<p>'.$key.'='.$val.' J!Value:'. $compare.' Matched:'.$allowLoad.'</p>';
				}
				if(!$allowLoad) break;
			}
			
			
			#Load the class only if the component is requested or forced
			if($allowLoad)
			{
				#Path Fix checks
				$loadClass=str_replace(array("/","\\"),DIRECTORY_SEPARATOR,$classDetails[1]); #Fix slash
				if($params["path"]!='absolute')
				{
					$loadClass=JPATH_ROOT.DIRECTORY_SEPARATOR.ltrim($loadClass,DIRECTORY_SEPARATOR);
				}

				if(JFile::exists($loadClass))
				{
					require_once($loadClass); #Load the custom class
					$html.='<p>File exists & Loaded:'.$loadClass.'</p>';
				}
				else
				{
					$html.='<p>File does not exists & Not loaded:'.$loadClass.'</p>';
				}
				$html.='<br/>'.print_r($params,true).'<br/>';
			}
			unset($classDetails,$params);
		}
		$html.='</pre></div>';
		if($debug) $this->app->enqueueMessage($html);
	}
		
	protected function getVar($key,$def=null)
	{
		$value=$this->app->input->get($key,$def);
		if($value==$def && isset($_REQUEST[$key]))
		{
			$filter = JFilterInput::getInstance();
			$value=$filter->clean($value, 'cmd');
		}
		return $value;
	}
	
	private function getParams($p)
	{
		$params=array('env'=>'site','event'=>'onAfterRoute','option'=>null,'view'=>null,'task'=>null,'layout'=>null,'controller'=>null,'path'=>null);
		if(empty($p)) return $params;
		$pp=explode(',',$p);
		foreach($pp as $np)
		{
			$vals=explode(":",$np,2);
			$params[trim(JString::strtolower($vals[0]))]=trim(JString::strtolower($vals[1]));
		}
		return $params;
	}
}